#!/bin/bash
if [ ! -f /usr/bin/dml ]; then echo "dml is required, install it and run me again"; exit 255; fi

. /etc/rc.d/init.d/functions
. /etc/rc.d/init.d/functions-live

INSTALLER_ID='$Id: installer.sh,v 1.117 2006/07/12 10:53:07 havner Exp $'
LIVECD_TOP="PLD LiveCD $INSTALLER_ID"
LIVECD_BANNER="PLD LiveCD Installer"

# give it $npart, it will return $gpart
normal_to_grub() {
        GROOT=`echo $npart| awk '{sub(/^hd|^sd|^c[0-9]d/,"");print $0}'`
        if echo $GROOT | grep -q '^[0-9][0-9]*$' ; then
                DRIVE=$GROOT
        else
                DRIVE=`echo $GROOT| awk '{sub(/p?[0-9]*$/,"");print $0}'`
        fi
        NR=`echo $GROOT| awk '{sub(/^([a-z]*|[0-9]*p?)/,"");print $0}'`
        NDRIVE=""

        if echo $DRIVE | grep -q '^[0-9][0-9]*$' ; then
                NDRIVE=$DRIVE
        else
                TR=`echo $DRIVE |tr "a-j" "0-9"`
                [ "${DRIVE}" != "$TR" ] && NDRIVE="$TR"
                TR=`echo $DRIVE |tr "k-u" "0-9"`
                [ "${DRIVE}" != "$TR" ] && NDRIVE="1${TR}"
                TR=`echo $DRIVE |tr "w-z" "0-3"`
                [ "${DRIVE}" != "$TR" ] && NDRIVE="2${TR}"
        fi

        if [ "${NR}" != "" ]; then
                NR=`expr $NR - 1`
                gpart="(hd${NDRIVE},$NR)"
        else
                gpart="(hd${NDRIVE})"
        fi
        unset TR NR NDRIVE GROOT DRIVE
}

# give it $npart, it will return $devpart
normal_to_dev() {
	devpart=`ls -l /dev/$npart | awk '{gsub(/,/,"",$5); printf "%0+2x%0+2x", $5, $6}'`
}

# Routine replacing 'hda1' with something nicer as 'First partition of first IDE disk'
identify_disk()
{
	case `echo $cp | cut -b 1` in
		h) bus=IDE;;
		s) bus=SCSI;;
	esac 
	case `echo $cp | cut -b 3` in
		a) busn=first;;
		b) busn=second;;
		c) busn=third;;
		d) busn=fourth;;
		e) busn=fifth;;
		f) busn=sixth;;
		g) busn=seventh;;
		h) busn=eighth;;
		i) busn=nineth;;
		j) busn=tenth;;
		k) busn=eleventh;;
	esac
	case `echo $cp | cut -b 4-` in
		1) partn=First;;
		2) partn=Second;;
		3) partn=Third;;
		4) partn=Fourth;;
		5) partn=Fifth;;
		6) partn=Sixth;;
		7) partn=Seventh;;
		8) partn=Eighth;;
		9) partn=Nineth;;
		10) partn=Tenth;;
		11) partn=Eleventh;;
	esac
	partid="$partn partition of $busn $bus disk ($cp)"
}

MENU_MAIN() {
	res=`dml -t "$LIVECD_TOP" << _EOF
	<br>
	$LIVECD_BANNER
	<br>
	Main menu
	<br>
	<br>
	<menu id=result res=1>
	<item id=partitioning>Partition settings
	<item id=bootloader>Bootloader configuration
	<item id=check>Check installation settings
	<item id=install>Installation
	<item id=quit>Quit installer
	</menu>
_EOF`
	eval `echo "$res" |grep result=`
}

MENU_PARTITIONING() {
	res=`dml -t "$LIVECD_TOP" << _EOF
	<br>
	$LIVECD_BANNER
	<br>
	Partition settings
	<br>
	<br>
	<menu id=result res=1>
	<item id=parted>Resize partitions with parted
	<item id=fdisk>Setup partitions with fdisk
	<item id=cfdisk>Setup partitions with cfdisk
	<item id=cip>Choose installation partition
	<item id=csp>Choose swap partition
	<item id=main>Back to main menu
	</menu>
_EOF`
	eval `echo "$res" |grep result=`
}

MENU_BOOTLOADER() {
	if [ $installpart ]; then
		echo "#This is autogenerated GRUB menu.lst 
# If it seems OK to you - do not edit it
# You can exit vi by pressing [esc] :wq [enter]
timeout 15
default 0
fallback 1 " > /tmp/install-grub-menu.lst
		echo >> /tmp/install-grub-menu.lst
		echo "title PLD Live" >> /tmp/install-grub-menu.lst
		npart=$installpart
		normal_to_grub
		normal_to_dev
		grootpart=$gpart
		PARAMS="root=$devpart"
		if getparam vga > /dev/null 2>&1; then
			PARAMS="$PARAMS vga=`getparam vga`"
		fi
		if getparam video > /dev/null 2>&1; then
			PARAMS="$PARAMS video=`getparam video`"
		fi
		echo "	root $gpart" >> /tmp/install-grub-menu.lst
		echo "	kernel /boot/vmlinuz $PARAMS" >> /tmp/install-grub-menu.lst
		echo "	initrd /boot/initrd" >> /tmp/install-grub-menu.lst

		for cp in `get_parts_other`; do
			identify_disk
			npart=$cp
			normal_to_grub
			echo >> /tmp/install-grub-menu.lst
			echo "title $partid" >> /tmp/install-grub-menu.lst
			echo "	root $gpart" >> /tmp/install-grub-menu.lst
			echo "	chainloader +1" >> /tmp/install-grub-menu.lst
		done
		if [ "$EDITOR" ]; then 
			"$EDITOR" /tmp/install-grub-menu.lst
		else
			vim /tmp/install-grub-menu.lst
		fi
		bootloader=Grub
		result=main
		choose_grub_hd
		npart=$grubdisk_nc
		normal_to_grub
		ginstallpart=$gpart
	else 
		msgboxmsg="Select install partition at first!"
		msgbox
		result=main
	fi
}

# Choose installation partition menu
MENU_CIP() {
	parts=""
	for cp in `get_parts_linux`; do
		identify_disk;
		parts="$parts <item id=$cp>$partid"
	done
	res=`dml -t "$LIVECD_TOP" << _EOF
	<br>
	$LIVECD_BANNER
	<br>
	Select installation partition
	<br>
	<br>
	<menu id=installpart res=1>
	$parts
	</menu>
_EOF`
	eval `echo "$res" |grep installpart=`
	result=fs
}

# Choose installation partition menu
MENU_CSP() {
	parts=""
	for cp in `get_parts_swap`; do
		identify_disk;
		parts="$parts <item id=$cp>$partid"
	done
	res=`dml -t "$LIVECD_TOP" << _EOF
	<br>
	$LIVECD_BANNER
	<br>
	Select swap partition
	<br>
	<br>
	<menu id=swappart res=1>
	$parts	
	<item id=none>No swap partition
	</menu>
_EOF`
	eval `echo "$res" |grep swappart=`
	result=main
}

MENU_FS() {
	res=`dml -t "$LIVECD_TOP" << _EOF
	<br>
	$LIVECD_BANNER
	<br>
	Select filesystem to use on $installpart
	<br>
	<br>
	<menu id=installpartfs res=1>
	<item id=xfs>XFS
	<item id=ext2>ext2
	<item id=ext3>ext3
	<item id=reiserfs>reiserfs
	<item id=unchanged>unchanged
	</menu>
	<br>
	<br>
_EOF`
	eval `echo "$res" |grep installpartfs=`
	result=main
}

MENU_CHECK() {
	not_done="";
	if [ $installpart ]; then
		m_installpart=$installpart
	else
		m_installpart="NOT SET!"   
		not_done="YES"
	fi
	if [ $installpartfs ]; then
		m_installpartfs=$installpartfs
	else
		m_installpartfs="NOT SET!"
		not_done="YES"
	fi
	if [ $bootloader ]; then
		m_bootloader=$bootloader
	else
		m_bootloader="NOT SET!"
		not_done="YES"
	fi
	if [ $ginstallpart ]; then
		m_ginstallpart=$ginstallpart
	else
		m_ginstallpart="NOT SET!"
		not_done="YES"
	fi

	if [ $swappart ]; then
		m_swappart=$swappart
	else
		m_swappart="NOT SET!"
		not_done="YES"
	fi
	if [ $swappart = "none" ]; then m_swappart="Not used"; fi 
	if [ ! $not_done ]; then installdata_ready="TRUE"; fi 
	res=`dml -t "$LIVECD_TOP" << _EOF
	<br>
	$LIVECD_BANNER
	<br>
	Check installation settings
	<br>
	<br>
	Install partition: $m_installpart
	<br>
	Install partition filesystem: $m_installpartfs
	<br>
	Bootloader: $m_bootloader
	<br>
	Bootloader install disk: $m_ginstallpart
	<br>
	<br>
	Swap partition: $m_swappart
	<br>
	<br>
	<button caption="Back to main menu" res=1>
_EOF`
	result=main
}

MENU_INSTALL() {

	if [ $installdata_ready ]; then
		clear

		show "Clearing"
		busy
		if grep "\/tmp\/dst" /proc/mounts > /dev/null 2>&1 ; then
			umount /tmp/dst
		fi
		if [ -d /tmp/dst ]; then
			rm -rf /tmp/dst
		fi
		ok

		if [ $installpartfs == "unchanged" ]; then
			show "Leaving current filesystem on $installpart"; ok
		else
			show "Creating $installpartfs on $installpart"; busy; echo
			if [ $installpartfs == "xfs" ]; then
				if mkfs.$installpartfs -f /dev/$installpart; then ok; else fail; fi
			else
				if mkfs.$installpartfs /dev/$installpart; then ok; else fail; fi
			fi
		fi

		show "Mounting filesystem"
		mkdir /tmp/dst
		if mount /dev/$installpart -t $installpartfs /tmp/dst; then ok; else fail; fi

		show "Creating base directories"
		busy
		for i in proc sys tmp selinux mnt media; do
			install -d /tmp/dst/$i
		done
		chmod 1777 /tmp/dst/tmp
#		for i in /mnt/*; do
#			install -d /tmp/dst/$i
#		done
		for i in /media/*; do
			install -d /tmp/dst/$i
		done
		ok

		show "Installing LiveCD to $installpart"
		busy
		for i in live/etc live/home live/var bin boot dev etc home lib opt root sbin usr var srv; do
#			install -d /tmp/dst/$i
			cp -dpRf /$i /tmp/dst/
		done
		install -d /tmp/dst/media/LiveCD/wallpapers
		install /media/LiveCD/wallpapers/* /tmp/dst/media/LiveCD/wallpapers/
		ok

		mount -t proc none /tmp/dst/proc

		show "Removing LiveCD style symlinks"
		busy
		for I in $SYMLINKS; do
			if [ -h /tmp/dst/$I ]; then
				rm -f /tmp/dst/$I
				mv -f /tmp/dst/live/$I /tmp/dst/$I
			fi
		done
#		ln -sf ../../../../etc/X11/fs /tmp/dst/usr/X11R6/lib/X11/fs
#		ln -sf ../../../../etc/X11/lbxproxy /tmp/dst/usr/X11R6/lib/X11/lbxproxy
#		ln -sf ../../../../etc/X11/proxymngr /tmp/dst/usr/X11R6/lib/X11/proxymngr
#		ln -sf ../../../../etc/X11/rstart /tmp/dst/usr/X11R6/lib/X11/rstart
#		ln -sf ../../../../etc/X11/twm /tmp/dst/usr/X11R6/lib/X11/twm
#		ln -sf ../../../../etc/X11/xinit /tmp/dst/usr/X11R6/lib/X11/xinit
#		ln -sf ../../../../etc/X11/xserver /tmp/dst/usr/X11R6/lib/X11/xserver
#		ln -sf ../../../../etc/X11/xsm /tmp/dst/usr/X11R6/lib/X11/xsm
#		ln -sf ../../../var/lib/xkb /tmp/dst/etc/X11/xkb/compiled
		rm -f /tmp/dst/usr/local
		mv -f /tmp/dst/var/local /tmp/dst/usr/
		install -d /tmp/dst/var/local
		ok

		show "Fixing OpenGL"
		busy; echo
		for I in $GLFILES; do
			rm -rf /tmp/dst/$I
		done
		if grep "nvidia" /etc/X11/xorg.conf > /dev/null 2>&1; then
			chroot /tmp/dst rpm -ihv /usr/src/X11-driver-nvidia*.rpm
		elif grep "fglrx" /etc/X11/xorg.conf > /dev/null 2>&1; then
			chroot /tmp/dst rpm --force -ihv --nodeps /usr/src/XFree86-driver-firegl*.rpm
			chroot /tmp/dst rpm --force -ihv --nodeps /usr/src/X11-OpenGL-core*.rpm
		else
			chroot /tmp/dst rpm -ihv /usr/src/X11-OpenGL-libGL*.rpm
			chroot /tmp/dst rpm -ihv /usr/src/X11-OpenGL-core*.rpm
		fi
		ok

		show "Fixing starting scripts"
		busy
		rpm -e livecd
		rpm -ivh /usr/scr/livecd-installed*.rpm
		chroot /tmp/dst/ /usr/sbin/alsactl store
#		cp -f /root/installer/rc.live.new /tmp/dst/etc/rc.d/rc.live
#		sed -i -e "s/#initlog/initlog/g" /tmp/dst/etc/rc.d/rc.sysinit
#		chroot /tmp/dst /sbin/chkconfig --del livecd
#		rm -f /tmp/dst/etc/rc.d/init.d/livecd /tmp/dst/etc/sysconfig/livecd
		for I in `ps x | grep "eth[0-9]\|usb[0-9]" | awk '{print $6}'`; do	# ugly
			sed -i -e "s/ONBOOT=no/ONBOOT=yes/g" /tmp/dst/etc/sysconfig/interfaces/ifcfg-$I
		done
		
		if getparam noX; then
			sed -i -e "s/^id:5/id:3/g" /tmp/dst/etc/inittab
		fi
		ok

		show "Creating new fstab"
		busy; echo
		echo "/dev/$installpart	/		$installpartfs		defaults		0 0" >> /tmp/dst/etc/fstab.new
		echo "/dev/${swappart}	swap		swap		defaults		0 0" >> /tmp/dst/etc/fstab.new
		mkswap /dev/${swappart}
		cat >> /tmp/dst/etc/fstab.new << EOF
none		/proc			proc	defaults,noauto,gid=17	0 0
none		/sys			sysfs	defaults,noauto		0 0
none		/proc/bus/usb		usbfs	defaults,noauto		0 0
none		/dev			devfs	defaults,noauto		0 0
none		/dev/pts		devpts	gid=5,mode=620		0 0
#none		/dev/shm		tmpfs	defaults		0 0

/dev/fd0	/media/floppy		auto	user,noauto		0 0
EOF
		echo "`grep cdrom /etc/fstab`" >> /tmp/dst/etc/fstab.new
		echo >> /tmp/dst/etc/fstab.new
		for I in `get_parts_linux`; do
			if [ "$I" != "$installpart" ]; then
				echo "/dev/${I}	/mnt/${I}_linux	auto		defaults,noauto,noatime	0 0" >> /tmp/dst/etc/fstab.new
				mkdir /tmp/dst/mnt/${I}_linux
			fi
		done
		echo >> /tmp/dst/etc/fstab.new
		for I in `get_parts_fat`; do
			echo "/dev/${I}	/mnt/${I}_fat	vfat		user,noauto,umask=002,uid=0,gid=28,iocharset=iso8859-2,codepage=852,exec,showexec 0 0" >> /tmp/dst/etc/fstab.new
			mkdir /tmp/dst/mnt/${I}_fat
		done
		for I in `get_parts_ntfs`; do
			echo "/dev/${I}	/mnt/${I}_ntfs	ntfs		user,noauto,umask=002,uid=0,gid=28,iocharset=iso8859-2 0 0" >> /tmp/dst/etc/fstab.new
			mkdir /tmp/dst/mnt/${I}_ntfs
		done
		mv -f /tmp/dst/etc/fstab.new /tmp/dst/etc/fstab
		ok

		show "Setting up bootloader"
		busy; echo
		cp -f /tmp/install-grub-menu.lst /tmp/dst/boot/grub/menu.lst
		chroot /tmp/dst /sbin/grub --batch << _EOF
		root $grootpart
		setup $ginstallpart
_EOF
		echo
		ok

		show "Generating initrd"
		busy
		V=`rpm --qf "%{VERSION}" -q kernel-grsecurity`
		R=`rpm --qf "%{RELEASE}" -q kernel-grsecurity`
		chroot /tmp/dst geninitrd -f /boot/initrd $V-$R
		ok

		show "Cleaning"
		busy
		rm -rf /tmp/dst/live
		rm -f /tmp/dst/usr/src/*.rpm
		echo -n > /tmp/dst/etc/motd
		umount /tmp/dst/proc
		umount /tmp/dst
		ok
		
		echo "Press ENTER"
		read NOTHING

		res=`dml -t "$LIVECD_TOP" << _EOF
	<br>
	$LIVECD_BANNER
	<br>
	<br>
	INSTALLATION DONE
	<br>
	Thank You for using PLD LiveCD Installer
	<br>
	Remember to set root password after booting
	<br>
	<button caption="Exit installator" res=1>
	<br>
	<br>
_EOF`
		result=install_done
	else 
		result=install_fail
	fi

}

choose_hd() {
	parts=""
	for cp in `get_disks`; do
		parts="$parts <item id=$cp>$cp"
	done
	res=`dml -t "$LIVECD_TOP" << _EOF
	<br>
	$LIVECD_BANNER
	<br>
	Choose disk to setup
	<br>
	<br>
	<menu id=disktopart res=1>
	$parts
	</menu>
_EOF`
	eval `echo "$res" |grep disktopart=`
}	

choose_grub_hd() {
	parts=""
	for cp in `get_disks`; do
		parts="$parts <item id=$cp>$cp"
	done
	parts="$parts <item id=$installpart>$installpart"
	res=`dml -t "$LIVECD_TOP" << _EOF
	<br>
	$LIVECD_BANNER
	<br>
	Choose disk You want to install GRUB on
	<br>
	<br>
	<menu id=grubdisk_nc res=1>
	$parts
	</menu>
_EOF`
	eval `echo "$res" |grep grubdisk_nc=`
}	

msgbox() {
	res=`dml -t "$LIVECD_TOP" << _EOF
	<br>
	$LIVECD_BANNER
	<br>
	<br>
	$msgboxmsg
	<br>
	<br>
	<button caption="Back to main menu" res=1>
	<br>
	<br>
_EOF`
	result=main
}

# res=`dml -t "$LIVECD_TOP" << _EOF
#	<br>
#	$LIVECD_BANNER
#	<br>
#	<br>
#	Before using, make sure you have the newest version of installer.
#	<br>
#	You can always update it by running 'cvs up' from installer dir.
#	<br>
#	<button caption="Start installer" res=1>
#	<br>
#	<br>
# _EOF`

current_menu=MAIN
installdata_ready=""
while [ 0 ]; do
	eval MENU_$current_menu
	case $result in
		partitioning) current_menu=PARTITIONING;;
		installsetup) current_menu=INSTALLSETUP;;
		install) current_menu=INSTALL;;
		bootloader) current_menu=BOOTLOADER;;
		main) current_menu=MAIN;;
		cip) current_menu=CIP;;
		csp) current_menu=CSP;;
		parted) choose_hd; parted /dev/$disktopart;;
		fdisk) choose_hd; fdisk /dev/$disktopart;;
		cfdisk) choose_hd; cfdisk /dev/$disktopart;;
		install_done) exit;;
		check) current_menu=CHECK;;
		fs) current_menu=FS;;
		install_fail) msgboxmsg="Check settings before actually installing"; msgbox; current_menu=MAIN;;
		quit) exit;;
	esac
done
